/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* SCCS ID: %W% %G% */
/*************************************************************************
 * FILE:      getScopeSetup.c
 * Author:    Claude Saunders
 * 
 * Purpose:   Program for retrieving scope setup data from an
 *            appropriately configured waveform record.
 *
 * Mods:      login mm/dd/yy: description
 *************************************************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "cadef.h"
#include "ezca.h"
#include "SDDS.h"          /* SDDS library header */

#define  VERSION   "V1.0"
#define  SDDS 1
#define  NELEMENTS 2058

/* Function Prototypes */
int process_cmdline_args(int argc, char *argv[], char *scopeid, char *file);
void usage(char *name);

int main(int argc, char *argv[]) {
  char scopeid[255];            /* name of scope */
  char setupfile[255];          /* name of file for setup data */
  char procpv[255];
  char readpv[255];
  unsigned char setupdata[NELEMENTS+100];
  short one = 1;
  int status;                  /* status of ezca operations */
  long i;
  SDDS_TABLE table;
#if (SDDS==1)
#else
  FILE *setupfp;
#endif
  process_cmdline_args(argc, argv, scopeid, setupfile);

  /* Form proper process variable names */
  strcpy(procpv,scopeid);
  strcat(procpv,":readSetupWF.PROC");
  strcpy(readpv,scopeid);
  strcat(readpv,":readSetupWF");

  ezcaSetTimeout(0.3);
  ezcaSetRetryCount(200);

  /* Read scope setup data */
  status = ezcaPut(procpv, ezcaShort, 1, &one);
  if (status) {
    fprintf(stderr, "%s: Unable to find desired scope %s\n",argv[0],scopeid);
    ca_task_exit();
    return(1);
  }
  sleep(1);
  status = ezcaGet(readpv, ezcaByte, NELEMENTS, setupdata);
  if (status) {
    fprintf(stderr, "%s: Unable to read scope setup data\n",argv[0]);
    ca_task_exit();
    return(1);
  }

#if (SDDS==1)
  if (!SDDS_InitializeOutput(&table, SDDS_BINARY, 1, "Scope Setup Data", 
			     "scope setup data", setupfile)) {
    fprintf(stderr,"%s: SDDS error: unable to initialize output file %s\n",
	    argv[0], setupfile);
    ca_task_exit();
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }

  if (SDDS_DefineColumn(&table, "scopeSetup", NULL, NULL, NULL, NULL,
			 SDDS_CHARACTER, 0) == -1) {
   fprintf(stderr,"%s: SDDS error: unable to define column %s\n",argv[0],
	   "scopeSetup");
    ca_task_exit();
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  } 

  if (!SDDS_WriteLayout(&table)) {
    fprintf(stderr,"%s: SDDS error: unable to write layout\n",argv[0]);
    ca_task_exit();
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }

  if (!SDDS_StartTable(&table,4096)) {
    fprintf(stderr,"%s: SDDS error: unable to start table\n",argv[0]);
    ca_task_exit();
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }

  for (i = 0 ; i < NELEMENTS ; i++) {
    if (!SDDS_SetRowValues(&table, (SDDS_SET_BY_NAME|SDDS_PASS_BY_VALUE), i,
			   "scopeSetup", setupdata[i], NULL)) {
      fprintf(stderr,"%s: SDDS error: unable to write row\n",argv[0]);
      ca_task_exit();
      SDDS_PrintErrors(stderr,SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors);
    }
  }

  if (!SDDS_WriteTable(&table)) {
    fprintf(stderr,"%s: SDDS error: unable to write table\n",argv[0]);
    ca_task_exit();
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }

  if (!SDDS_Terminate(&table)) {
    fprintf(stderr,"%s: SDDS error: unable to terminate SDDS\n",argv[0]);
    ca_task_exit();
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }
#else
  setupfp = fopen(setupfile,"w");
  if (setupfp == NULL) {
    fprintf(stderr,"%s: error: unable to open %s for writing\n",argv[0],
	    setupfile);
    ca_task_exit();
    return(1);
  }
  if (fwrite(setupdata, sizeof(unsigned char), NELEMENTS, setupfp) != 
      NELEMENTS) {
    fprintf(stderr,"%s: error: unable to write data to %s\n",argv[0],
	    setupfile);
    ca_task_exit();
    return(1);
  }
  fclose(setupfp);
#endif
  ca_task_exit();
  return(0);

}
/*************************************************************************
 * FUNCTION : process_cmdline_args()
 * PURPOSE  : Get command line arguments and set flags appropriately.
 * ARGS in  : argc, argv - standard unix
 * ARGS out : scopeid - name of scope
 *            setupfile - name of file to dump scope setup into
 * GLOBAL   : nothing
 * RETURNS  : 0
 ************************************************************************/  
int
process_cmdline_args(int argc, char *argv[], char *scopeid, char *setupfile)
{
  int cmderror = 0;

  *scopeid = '\0';
  *setupfile = '\0';

  /* Process command line options */
  if (argc != 3)
    cmderror++;
  else {
    strcpy(scopeid,argv[1]);
    strcpy(setupfile,argv[2]);
    }

  if (cmderror) {
    usage(argv[0]);
    exit(1);
  }
  return(0);
}

/*************************************************************************
 * FUNCTION : usage()
 * PURPOSE  : Describes command line options available to client.
 * ARGS in  : char *name - name of executable program
 * ARGS out : none
 * GLOBAL   : nothing
 * RETURNS  : none
 ************************************************************************/  
void usage(char *name)
{
  fprintf(stderr,"Get Scope Setup Utility %s\n",VERSION);
  fprintf(stderr,"----------------------------\n");
  fprintf(stderr,"usage: %s <scope-id> <setup-filename>\n",name);
  fprintf(stderr,"purpose:\tReads the current scope setup from <scope-id>.\n");
  fprintf(stderr,"\t\tBinary setup data is written to <setup-filename>.\n");
}

