/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* SCCS ID: %W% %G% */
/*************************************************************************
 * FILE:      getScopeWaveform.c
 * Author:    Claude Saunders
 * 
 * Purpose:   Program for retrieving a scope waveform from an 
 *            appropriately configured waveform record. Also requires
 *            records which contain the scaling data (Xincr, Yorigin, etc).
 *
 * Mods:      login mm/dd/yy: description
 *************************************************************************/

#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <limits.h>
#include <time.h>
#if !defined(vxWorks)
#include <sys/timeb.h>
#endif
/*
#include <sys/param.h>
*/
#include "cadef.h"
#include "ezca.h"
#include "mdb.h"           /* MDB library header  */
#include "SDDS.h"          /* SDDS library header */
#include "scan.h"
#include "match_string.h"
#include <ctype.h>

#if defined(linux) || !defined(HUGE)
#define  MAXFLOAT        ((float)3.40282346638528860e+38)
#define  HUGE            MAXFLOAT
#endif

#define  VERSION   "V1.3"
#define  MAXELEMENTS 32767
#define  EXPECTED_TABLE_ROWS 1000000

/* Hardcoded path (sorry, time constraints necessitated quick fix */
#define  SCOPETMP_DIR "/home/helios4/scopetmp"

/* #define  DEBUG */

#define SET_SIGNAL_NAME 0
#define SET_DESCRIPTION 1
#define SET_MPL_LABELS 2
#define SET_INITIAL_T 3
#define SET_SINGLE_PRECISION 4
#define SET_SEGMENT_INTERVAL 5
#define N_OPTIONS 6

static char *option[N_OPTIONS] = {
  "signal_name", "description", "mpl_labels",
  "initial_t", "single_precision", "segment_interval"
  };

char *USAGE = "usage: getScopeWaveform <scopeid> <chan> <outputfile>\
 -signal_name=<name> [-description=<text>,<contents>] [-mpl_labels=<title>,\
<topline>] [-initial_t=<value>] [-single_precision]\
 [-segment_interval=<time-in-secs>]\n\
Utility to fetch scope data from waveform and preamble records, and write\
 out an SDDS file. <scopeid> must have appropriate records defined.\n\
    <scopeid> name of scope\n\
    <chan>    c1, c2, c3, c4, m1, m2, m3, or m4\n";

/******* Function Prototypes ********/
int
process_cmdline_args(int argc, char *argv[], char *scopeid, char *scopechan,
		     char *waveformfile, char **signal_name, 
		     char **descrip_text, char **descrip_contents,
		     char **mpl_title, char **mpl_topline, 
		     double *initial_t, long *single_precision,
		     double *segment_interval);
long define_2d_parameters(SDDS_TABLE *table, char *parameter, char *variable, char *units);
long set_2d_parameters(SDDS_TABLE *table, char *variable, double initial, long dimension, double increment);

/*************************************/
/********** Main Block ***************/
/*************************************/
int main(int argc, char *argv[]) {
                                /* Command line argument data */
  char scopeid[255];            /* name of scope */
  char scopechan[255];          /* scope or memory channel name */
  char waveformfile[255];       /* name of file for waveform data */
  char *signal_name, *mpl_title, *mpl_topline;
  char *descrip_text, *descrip_contents;
  double initial_t, segment_interval;
  long single_precision;

  char *currentdir;             /* current execution directory */
  long maxpath;                 /* Max length of path name */
  char procpv[255];             /* PV to initiate waveform collection */
  char pathpv[255];             /* PV to put current path to */
  char rawdatafnpv[255];        /* PV with name of raw data file written */
  char rawdatafn[255];          /* name of raw data file */
  char rawdatapath[255];        /* full path to raw data file */
  FILE *rawdatafp;
  uint32_t nelements;      /* # of data items from raw data file header */
  unsigned short *elements;     /* Data elements from raw file */
  double *data=NULL, *time=NULL;          /* scaled data elements as doubles */
  float *sp_data=NULL, *sp_time=NULL;     /* scaled data elements as floats  */
  double xinc, xor, xref, yinc, yor, yref;
  double points, count;         /* points per segment, number of segments */
  char xincpv[255], xorpv[255], xrefpv[255];
  char yincpv[255], yorpv[255], yrefpv[255];
  char pointspv[255], countpv[255];
  short one = 1;
  int status;                  /* status of ezca operations */
  long i;
  char junk[255];
  SDDS_TABLE table;
  double startTime; /*start time of collecting waveform data */
  char rawdataFile[1025]; /*raw data file full name */

  /* Get path limit from operating system. */
  if ((maxpath = pathconf("/",_PC_PATH_MAX)) < 0)
     maxpath = 1024;
  else
     maxpath++;
  currentdir = (char *)malloc(sizeof(char)*maxpath);

  process_cmdline_args(argc, argv, scopeid, scopechan, waveformfile, 
		       &signal_name, &descrip_text, &descrip_contents,
		       &mpl_title, &mpl_topline, &initial_t, &single_precision,
		       &segment_interval);

  /* Form proper process variable names */
  strcpy(xincpv,scopeid);
  strcat(xincpv,":XincAI");
  strcpy(xorpv,scopeid);
  strcat(xorpv,":XorAI");
  strcpy(xrefpv,scopeid);
  strcat(xrefpv,":XrefAI");

  strcpy(yincpv,scopeid);
  strcat(yincpv,":YincAI");
  strcpy(yorpv,scopeid);
  strcat(yorpv,":YorAI");
  strcpy(yrefpv,scopeid);
  strcat(yrefpv,":YrefAI");

  strcpy(pointspv,scopeid);
  strcat(pointspv,":pointsAI");
  strcpy(countpv,scopeid);
  strcat(countpv,":countAI");

  strcpy(procpv,scopeid);
  strcpy(pathpv,scopeid);
  strcpy(rawdatafnpv,scopeid);

  if (!strcmp(scopechan,"c1") || !strcmp(scopechan,"C1")) {
    strcat(procpv,":chan1WaveWF.PROC");
    strcat(pathpv,":chan1FileDL.DNAM");
    strcat(rawdatafnpv,":chan1FileDL.FNAM");

  } else if (!strcmp(scopechan,"c2") || !strcmp(scopechan,"C2")) {
    strcat(procpv,":chan2WaveWF.PROC");
    strcat(pathpv,":chan2FileDL.DNAM");
    strcat(rawdatafnpv,":chan2FileDL.FNAM");

  } else if (!strcmp(scopechan,"c3") || !strcmp(scopechan,"C3")) {
    strcat(procpv,":chan3WaveWF.PROC");
    strcat(pathpv,":chan3FileDL.DNAM");
    strcat(rawdatafnpv,":chan3FileDL.FNAM");

  } else if (!strcmp(scopechan,"c4") || !strcmp(scopechan,"C4")) {
    strcat(procpv,":chan4WaveWF.PROC");
    strcat(pathpv,":chan4FileDL.DNAM");
    strcat(rawdatafnpv,":chan4FileDL.FNAM");

  } else if (!strcmp(scopechan,"m1") || !strcmp(scopechan,"M1")) {
    strcat(procpv,":mem1WaveWF.PROC");
    strcat(pathpv,":mem1FileDL.DNAM");
    strcat(rawdatafnpv,":mem1FileDL.FNAM");

  } else if (!strcmp(scopechan,"m2") || !strcmp(scopechan,"M2")) {
    strcat(procpv,":mem2WaveWF.PROC");
    strcat(pathpv,":mem2FileDL.DNAM");
    strcat(rawdatafnpv,":mem2FileDL.FNAM");

  } else if (!strcmp(scopechan,"m3") || !strcmp(scopechan,"M3")) {
    strcat(procpv,":mem3WaveWF.PROC");
    strcat(pathpv,":mem3FileDL.DNAM");
    strcat(rawdatafnpv,":mem3FileDL.FNAM");

  } else if (!strcmp(scopechan,"m4") || !strcmp(scopechan,"M4")) {
    strcat(procpv,":mem4WaveWF.PROC");
    strcat(pathpv,":mem4FileDL.DNAM");
    strcat(rawdatafnpv,":mem4FileDL.FNAM");
  } 

  ezcaSetTimeout(0.3);
  ezcaSetRetryCount(100);

  /* Put current path in datalog record */
  if (getcwd(currentdir,(size_t)maxpath) == NULL) {
    fprintf(stderr,"%s: Unable to get current working directory: %s\n",argv[0],
	    currentdir);
    ca_task_exit();
    return(1);
  }

  /* Put SCOPETMP_DIR path in datalog record */
  status = ezcaPut(pathpv, ezcaByte, strlen(SCOPETMP_DIR), SCOPETMP_DIR);
  if (status) {
    fprintf(stderr, "%s: Unable to find desired scope %s\n",argv[0],scopeid);
    ca_task_exit();
    return(1);
  }
  strcpy(rawdatapath,SCOPETMP_DIR);
  
  /* Get raw data filename that will be written in cwd */
  status = ezcaGet(rawdatafnpv, ezcaString, 1, rawdatafn);
  if (status) {
    fprintf(stderr, "%s: Unable to find desired scope %s\n",argv[0],scopeid);
    ca_task_exit();
    return(1);
  }
  strcpy(rawdataFile,rawdatapath);
  strcat(rawdataFile,"/");
  strcat(rawdataFile,rawdatafn);
 
  /*remove the existing file before collecting data */
  if (fexists(rawdataFile)) {
    remove(rawdataFile);
    sleep(1);
  }
  /* Process waveform record */
  status = ezcaPut(procpv, ezcaShort, 1, &one);
  /* Wait for the file to be written 
   * XXX Should be smarter here!
   */
  if (status) {
    fprintf(stderr, "%s: Unable to find desired scope %s\n",argv[0],scopeid);
    ca_task_exit();
    return(1);
  }
  startTime=getTimeInSecs();
  while ((getTimeInSecs()-startTime)<30.0) {
    if (fexists(rawdataFile))
      break;
    sleep(1);
  }
  if (!fexists(rawdataFile)) {
    fprintf(stderr,"Raw data has not be collected after 30 seconds, something may be wrong with the waveform data collection.\n");
    ca_task_exit();
    return(1);
  }

  /*sleep(4);*/
  
  /* Collect waveform preamble information */
  ezcaStartGroup();
  ezcaGet(xincpv, ezcaDouble, 1, &xinc);
  ezcaGet(xorpv, ezcaDouble, 1, &xor);
  ezcaGet(xrefpv, ezcaDouble, 1, &xref);
  ezcaGet(yincpv, ezcaDouble, 1, &yinc);
  ezcaGet(yorpv, ezcaDouble, 1, &yor);
  ezcaGet(yrefpv, ezcaDouble, 1, &yref);
  ezcaGet(pointspv, ezcaDouble, 1, &points);
  ezcaGet(countpv, ezcaDouble, 1, &count);
  status = ezcaEndGroup();
  if (status) {
    fprintf(stderr, 
	    "%s: Unable to read %s waveform preamble\n",argv[0],scopeid);
    ca_task_exit();
    return(1);
  }
#ifdef DEBUG
  fprintf(stdout,"\nWaveform Preamble:\n");
  fprintf(stdout,"xinc: %le xor: %le xref: %le\n",xinc,xor,xref);
  fprintf(stdout,"yinc: %le yor: %le yref: %le\n",yinc,yor,yref);
  fprintf(stdout,"points: %le count: %le\n\n",points, count);
#endif

  /* Open raw data file from SCOPETMP_DIR */
  strcat(rawdatapath,"/");
  strcat(rawdatapath,rawdatafn);
  rawdatafp = fopen(rawdatapath,"r");
  if (rawdatafp == NULL) {
    fprintf(stderr,"%s: Unable to open raw data file written by IOC\n",
	    argv[0]);
    ca_task_exit();
    return(1);
  }

  /* skip over first two items */
  if (!fread(junk, sizeof(uint32_t), 2, rawdatafp)) {
    fprintf(stderr,"%s: Unable to read from raw data file\n",argv[0]);
    /* Remove raw data file */
    fclose(rawdatafp);
    unlink(rawdatafn);
    ca_task_exit();
    return(1);
  }
  if (!fread((char *)(&nelements), sizeof(uint32_t), 1, rawdatafp)) {
    fprintf(stderr,"%s: Unable to read from raw data file\n",argv[0]);
    /* Remove raw data file */
    fclose(rawdatafp);
    unlink(rawdatafn);
    ca_task_exit();
    return(1);
  }
  if (!SDDS_IsBigEndianMachine()) {
    SDDS_SwapULong(&nelements);
  }
#ifdef DEBUG
  fprintf(stdout,"# of elements in raw data file %d\n",nelements);
#endif
  if (nelements != (points*count)) {
    fprintf(stderr,"%s: raw data doesn't match segment count and size\n",
	    argv[0]);
    /* Remove raw data file */
    fclose(rawdatafp);
    unlink(rawdatafn);
    ca_task_exit();
    return(1);
  }

  /* Allocate space for data points in file */
  elements = (unsigned short *)malloc(nelements * sizeof(unsigned short));
  if (!single_precision) {
    data = (double *)malloc(nelements * sizeof(double));
    time = (double *)malloc(nelements * sizeof(double));
  } else {
    sp_data = (float *)malloc(nelements * sizeof(float));
    sp_time = (float *)malloc(nelements * sizeof(float));
  }

  if (!SDDS_InitializeOutput(&table, SDDS_BINARY, 0, descrip_text,
			     descrip_contents, waveformfile)) {
    fprintf(stderr,"%s: SDDS error: unable to initialize output file %s\n",
	    argv[0], waveformfile);
    /* Remove raw data file */
    fclose(rawdatafp);
    unlink(rawdatafn);
    ca_task_exit();
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }

  if (mpl_title && 
      (SDDS_DefineParameter(&table, "mplTitle", NULL,NULL,NULL,NULL,
			    SDDS_STRING, mpl_title)<0 ||
       SDDS_DefineParameter(&table, "mplTopline", NULL,NULL,NULL,NULL,
			    SDDS_STRING, mpl_topline)<0)) {
    ca_task_exit();
    SDDS_PrintErrors(stderr, (SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }
  if ((long)count!=1) {
    if (!define_2d_parameters(&table, "Variable1Name", "Segment", NULL)<0 ||
	!define_2d_parameters(&table, "Variable2Name", "t", "s")<0) {
      /* Remove raw data file */
      fclose(rawdatafp);
      unlink(rawdatafn);
      ca_task_exit();
      return(1);
    }
  }

  if (SDDS_DefineColumn(&table, "t", NULL, "s", NULL, NULL,
			 (single_precision?SDDS_FLOAT:SDDS_DOUBLE), 0) == -1) {
    fprintf(stderr,"%s: SDDS error: unable to define column %s\n",argv[0],
	   "t");
    /* Remove raw data file */
    fclose(rawdatafp);
    unlink(rawdatafn);
    ca_task_exit();
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  } 
  if (SDDS_DefineColumn(&table, signal_name, NULL, "V", NULL, NULL,
			 (single_precision?SDDS_FLOAT:SDDS_DOUBLE), 0) == -1) {
    fprintf(stderr,"%s: SDDS error: unable to define column %s\n",argv[0],
	    signal_name);
    /* Remove raw data file */
    fclose(rawdatafp);
    unlink(rawdatafn);
    ca_task_exit();
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  } 
  
  if (!SDDS_WriteLayout(&table)) {
    fprintf(stderr,"%s: SDDS error: unable to write layout\n",argv[0]);
    /* Remove raw data file */
    fclose(rawdatafp);
    unlink(rawdatafn);
    ca_task_exit();
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }

  if (!SDDS_StartTable(&table,EXPECTED_TABLE_ROWS)) {
    fprintf(stderr,"%s: SDDS error: unable to start table\n",argv[0]);
    /* Remove raw data file */
    fclose(rawdatafp);
    unlink(rawdatafn);
    ca_task_exit();
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }

  /* Read waveform from raw data file */
  if (!fread((char *)elements,sizeof(unsigned short),nelements,rawdatafp)) {
    fprintf(stderr,"%s: Unable to read data points from raw data file\n",
	    argv[0]);
    /* Remove raw data file */
    fclose(rawdatafp);
    unlink(rawdatafn);
    ca_task_exit();
    return(1);
  }
  if (!SDDS_IsBigEndianMachine()) {
    for (i=0 ; i < nelements ; i++) {
      SDDS_SwapUShort(&elements[i]);
    }
  }
  /* Remove raw data file */
  fclose(rawdatafp);
  unlink(rawdatafn);

  if (!single_precision) {
    if (initial_t == HUGE)
      initial_t = -xref * xinc + xor;
    for (i=0 ; i < nelements ; i++) {
      data[i] = ((double)elements[i]-yref) * yinc + yor;
      if (count != 1 && i && i%(long)points==0)
	initial_t += segment_interval - points * xinc;
      time[i] = i * xinc + initial_t;
    }
  } else {
    if (initial_t == HUGE)
      initial_t = -xref * xinc + xor;
    for (i=0 ; i < nelements ; i++) {
      sp_data[i] = ((float)elements[i]-yref) * yinc + yor;
      if (count != 1 && i && i%(long)points==0)
	initial_t += segment_interval - points * xinc;
      sp_time[i] = i * xinc + initial_t;
    }
  }

  if ((long)count!=1) {
    if (!set_2d_parameters(&table, "Segment", 0.0, (long)count, 1.0) ||
	!set_2d_parameters(&table, "t", initial_t, (long)points, xinc))
      ca_task_exit();
      return(1);
  }

  if (!SDDS_SetColumn(&table, SDDS_SET_BY_NAME,
		      (single_precision?(void*)sp_time:(void*)time),
		      nelements, "t")) {
    fprintf(stderr,"%s: SDDS error: unable to write row\n",argv[0]);
    ca_task_exit();
    SDDS_PrintErrors(stderr,SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors);
  }
  if (!SDDS_SetColumn(&table, SDDS_SET_BY_NAME,
		      (single_precision?(void*)sp_data:(void*)data), 
		      nelements, signal_name)) {
    fprintf(stderr,"%s: SDDS error: unable to write row\n",argv[0]);
    ca_task_exit();
    SDDS_PrintErrors(stderr,SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors);
  }
  
  if (!SDDS_WriteTable(&table)) {
    fprintf(stderr,"%s: SDDS error: unable to write table\n",argv[0]);
    ca_task_exit();
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }

  if (!SDDS_Terminate(&table)) {
    fprintf(stderr,"%s: SDDS error: unable to terminate SDDS\n",argv[0]);
    ca_task_exit();
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }
  ca_task_exit();
  return(0);
}
/*************************************************************************
 * FUNCTION : process_cmdline_args()
 * PURPOSE  : Get command line arguments.
 * ARGS in  : argc, argv - standard unix
 * ARGS out : 
 * GLOBAL   : nothing
 * RETURNS  : 0
 ************************************************************************/  
int
process_cmdline_args(int argc, char *argv[], char *scopeid, char *scopechan,
		     char *waveformfile, char **signal_name, 
		     char **descrip_text, char **descrip_contents,
		     char **mpl_title, char **mpl_topline, 
		     double *initial_t, long *single_precision,
		     double *segment_interval)
{
  SCANNED_ARG *scanned;
  int i;

  *scopeid = 0;
  *scopechan = 0;
  *waveformfile = 0;
  *signal_name = "V";
  *descrip_text = *descrip_contents = *mpl_title = *mpl_topline = NULL;
  *initial_t = HUGE;
  *single_precision = 0;
  *segment_interval = 0;

  argc = scanargs(&scanned, argc, argv);
  if (argc < 4) {
    fprintf(stderr,"Get Scope Waveform Utility %s\n",VERSION);
    fprintf(stderr,"-------------------------------\n");
    bomb(NULL, USAGE);
  }

  for (i=1 ; i<argc ; i++) {
    if (scanned[i].arg_type==OPTION) {
      /* process options here */
      switch (match_string(scanned[i].list[0], option, N_OPTIONS, 0)) {
      case SET_SIGNAL_NAME:
	if (scanned[i].n_items !=2)
	  bomb("invalid -signal_name syntax", USAGE);
	*signal_name = scanned[i].list[1];
	break;
      case SET_DESCRIPTION:
	if (scanned[i].n_items !=3)
	  bomb("invalid -description syntax", USAGE);
	*descrip_text = scanned[i].list[1];
	*descrip_contents = scanned[i].list[2];
	break;
      case SET_MPL_LABELS:
	if (scanned[i].n_items !=3)
	  bomb("invalid -mpl_labels syntax", USAGE);
	*mpl_title = scanned[i].list[1];
	*mpl_topline = scanned[i].list[2];
	break;
      case SET_INITIAL_T:
	if (scanned[i].n_items !=2 ||
	    sscanf(scanned[i].list[1], "%lf", initial_t)!=1)
	  bomb("invalid -initial_t syntax", USAGE);
	break;
      case SET_SINGLE_PRECISION:
	*single_precision = 1;
	break;
      case SET_SEGMENT_INTERVAL:
	if (scanned[i].n_items !=2 ||
	    sscanf(scanned[i].list[1], "%lf", segment_interval)!=1)
	  bomb("invalid -segment_interval syntax", USAGE);
	break;	
      default:
	bomb("invalid option seen", USAGE);
	break;
      }
    } else {
      if (!(*scopeid))
	strcpy(scopeid,scanned[i].list[0]);
      else if (!(*scopechan))
	strcpy(scopechan, scanned[i].list[0]);
      else if (!(*waveformfile))
	strcpy(waveformfile, scanned[i].list[0]);
      else
	bomb("too many required arguments", USAGE);
    }
  }
  if (!(*scopeid))
    bomb("Scope id not seen", USAGE);
  if (!(*scopechan))
    bomb("Scope channel not seen", USAGE);
  if (!(*waveformfile))
    bomb("Output file not seen", USAGE);
  if (!(*signal_name))
    bomb("-signal_name not seen", USAGE);

  return(0);
}

long define_2d_parameters(SDDS_TABLE *table, char *parameter, 
			  char *variable, char *units)
{
  char buffer[255];
  SDDS_DefineParameter(table, parameter, NULL, NULL, NULL, NULL, SDDS_STRING, variable);
  sprintf(buffer, "%sMinimum", variable);
  SDDS_DefineParameter(table, buffer, NULL, units, NULL, NULL, SDDS_DOUBLE, NULL);
  sprintf(buffer, "%sInterval", variable);
  SDDS_DefineParameter(table, buffer, NULL, units, NULL, NULL, SDDS_DOUBLE, NULL);
  sprintf(buffer, "%sDimension", variable);
  SDDS_DefineParameter(table, buffer, NULL, NULL, NULL, NULL, SDDS_LONG, NULL);
  if (SDDS_NumberOfErrors()) {
    SDDS_PrintErrors(stderr, SDDS_VERBOSE_PrintErrors);
    return(0);
  }
  return(1);
}

long set_2d_parameters(SDDS_TABLE *table, char *variable, double initial, 
		       long dimension, double increment)
{
  char buffer[255];
  sprintf(buffer, "%sMinimum", variable);
  SDDS_SetParameters(table, SDDS_SET_BY_NAME|SDDS_PASS_BY_VALUE, buffer, initial, NULL);
  sprintf(buffer, "%sInterval", variable);
  SDDS_SetParameters(table, SDDS_SET_BY_NAME|SDDS_PASS_BY_VALUE, buffer, increment, NULL);
  sprintf(buffer, "%sDimension", variable);
  SDDS_SetParameters(table, SDDS_SET_BY_NAME|SDDS_PASS_BY_VALUE, buffer, dimension, NULL);
  if (SDDS_NumberOfErrors()) {
    SDDS_PrintErrors(stderr, SDDS_VERBOSE_PrintErrors);
    return(0);
  }
  return(1);
}


